/*
 * Field.cpp
 *
 *  Created on: Dec 4, 2011
 *      Author: janez
 */

#include "Field.h"

Field::Field() {
	init();
}

Field::~Field() {
	//
}


/**
 * Initialize field object.
 */
void Field::init() {
	type = FIELD_TYPE_E;
	angle_yz = angle_xy = 0;
	abs_value = 0;
}


/**
 * Compute force on given particle caused by field.
 * @return force on given particle in 3 directions.
 */
Vector3 Field::computeForce( Particle *part ) {
	Vector3 F(0.0, 0.0, 0.0);


	if( type == Field::FIELD_TYPE_B ) {
		// magnetic field
		if( angle_xy == 90 && angle_yz == 0 ) {
			// magnetic field in direction of positive Y axis
			F = force_B_axis_Y( 1, part );

		} else if( angle_xy == 90 && angle_yz == 90 ) {
			// magnetic field in direction of positive Z axis
			F = force_B_axis_Z( 1, part );

		} else {
			// general case;
		}

	} else if( type == Field::FIELD_TYPE_E ) {
		// electric field is not implemented yet; return 0
		F.set(0.0, 0.0, 0.0);
	}


	return F;
}



/**
 * Compute force on given particle in case of magnetic field is oriented in Y direction.
 * First parameter defined direction (positive or negative).
 *
 * @return force on given particle in 3 directions.
 */
Vector3 Field::force_B_axis_Y( int direction, Particle *pt ) {
	Vector3 F(0.0, 0.0, 0.0);

	// compute force
	F.set_e3( direction * pt->get_q() / pt->get_m() * pt->get_v().e1() * this->abs_value );

	return F;
}


/**
 * Compute force on given particle in case of magnetic field is oriented in Z direction.
 * First parameter defined direction (positive or negative).
 *
 * @return force on given particle in 3 directions.
 */
Vector3 Field::force_B_axis_Z( int direction, Particle *pt ) {
	Vector3 F(0.0, 0.0, 0.0);

	// compute force; negative sign because in case of v_x is positive and B is in Z+ direction,
	// force is in Y- direction
	F.set_e2( -1 * direction * pt->get_q() / pt->get_m() * pt->get_v().e1() * this->abs_value );

	return F;
}

