/*
 * Grid.cpp
 *
 *  Created on: Aug 18, 2011
 *      Author: janez
 */

#include "Grid.h"
#include "Vector.h"

#include <string>
#include <sstream>
#include <iostream>
#include <vector>

using namespace std;


Grid::Grid() {
	init();
}

Grid::~Grid() {
}


/**
 * Initialize object.
 */
void Grid::init(){
	r0.set(0.,0.,0.);
	r1.set(0.,0.,0.);
	n[0] = n[1] = n[2] = 0;
}

/**
 * Return position of given grid point (i_point) in given direction (i_dir).
 */
double Grid::get_x( int i_dir, long int i_point ) {
	if( 0 <= i_point && i_point < n[i_dir] && 0 <= i_dir && i_dir < 3 ) {
		return r0.e(i_dir+1) + i_point * 1. / n[i_dir] * (r1.e(i_dir+1) - r0.e(i_dir+1));
	} else {
		return 0;
	}
}



/**
 * Display grid data.
 */
void Grid::dump(int level) {
	string offset = "";
	Vector3 size = r1 - r0;

	for( int i=0 ; i < level ; i++ ) {
		offset += "  ";
	}

	cout << endl << offset << "Grid: " << endl;
	cout << offset << "  Start point (x,y,z): " << r0.e1() << ", " << r0.e2() << ", " << r0.e3() << "; " ;
	cout << offset << "  end point (x,y,z): " << r1.e1() << ", " << r1.e2() << ", " << r1.e3() << endl ;
	cout << offset << "  x-size: " << size.e1() << ", cells: " << n[0] << "; ";
	cout << "y-size: " << size.e2() << ", cells: " << n[1] << "; ";
	cout << "z-size: " << size.e3() << ", cells: " << n[2] << endl;
}


/**
 * Check if minimal set of data is defined.
 * Return true if yes, and false of not enough data is defined.
 */
bool Grid::defined(bool dispErrors) {
	bool ret = false;
	Vector3 size;
	vector<string> mess;
	stringstream tmp;

	// calculate size of grid; this should not be zero
	size = r1 -r0;

	if( size.magnitude() > 0 ) {
		// size of grid space is not zero; OK
		ret = true;
	} else {
		// dimension of grid not defined
		mess.push_back("Grid dimensions are not defined.");
	}

	if( ret && ( (size.e1() != 0 && n[0] > 0) || (size.e2() != 0 && n[1] > 0) ||
					(size.e3() != 0 && n[2] > 0) ) ) {
		// number of cells is defined in direction that grid is defined
		ret = true;
	} else {
		// no, grid is not defined OK.
		mess.push_back("There seems that cells are defined in direction that grid is zero. Check it.");
		tmp << "x-size: " << size.e1() << ", cells:" << n[0] << "; ";
		tmp << "y-size: " << size.e2() << ", cells:" << n[1] << "; ";
		tmp << "z-size: " << size.e3() << ", cells:" << n[2];
		mess.push_back( tmp.str() );

		ret = false;
	}

	if( dispErrors && mess.size() > 0  ) {
		// display errors
		for( unsigned int i=0 ; i < mess.size() ; i++ ) {
			cout << mess[i] << endl;
		}
	}

	// clear message(s)
	mess.clear();

	return ret;
}

/**
 * Return cell index for given position (x) and in given direction.
 * If position is outside grid limits, returns -1 (before lower limit), -2 (after upper limit)
 */
int Grid::get_cell( int i_dir, double x ) {
	int cell_i = -1;
	double x_min, x_max;

	if( 0 <= i_dir && i_dir < 3 ) {
		// direction is OK; check dimension
		x_min = this->r0.e(i_dir+1);
		x_max = this->r1.e(i_dir+1);

		if( x < x_min ) {
			// given position is before minimum
			cell_i = -1;
		} else if( x > x_max ) {
			// given position is above maximum
			cell_i = -2;
		} else {
			// compute cell
			if( IS_ZERO(x-x_max) ) {
				// given position is on max limit; last cell
				cell_i = this->n[i_dir] - 1;
			} else {
				// compute
				cell_i = (int) ( n[i_dir] * (x - x_min) / (x_max - x_min)  );
			}
		}
	}

	return cell_i;
}


