/*
 * InitialLoad.cpp
 *
 *  Created on: 2011-08-15
 *      Author: janez
 */

#include "InitialLoad.h"

#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "RandomGenerator.h"

using namespace std;


InitialLoad::InitialLoad() {
	init();
}

InitialLoad::~InitialLoad() {

}


/**
 * Initialize source object.
 */
void InitialLoad::init() {
	Source::init();
	// define step type as NONE to avoid printing error for not defining dStep/dt
	type = TYPE_DELTA_NONE;
}


/**
 * Inject particles if it is needed.
 * plist -- particle list
 * in_region -- region in which to inject particle(s)
 * in_index -- species index that we have to inject
 * t -- current simulation time
 * step -- current simulation time
 */
ParticleList InitialLoad::inject( ParticleList& plist, Tree &in_tree, SpatialRegion& in_region, int in_sp_index, double t, long int step ) {
	Vector3 v, r, v_comp, r_comp;
	Particle new_p;
	Compute_green compute;
	ParticleList retList;

	// initialize return list
	retList.init();


	if( in_sp_index != this->sp_index ) {
		// OK, current source does not add requested species
		return retList;
	}

#if 1
	// use method that is used also for source
	retList = doInjectParticles( plist, in_tree, in_region, t, step);

#else
	// use faster version -- in initial load there are no particles in system so there is no
	// need to compute forces on particles (they are 0) and new positions.
	// forces and positions are computed after all particles are injected

	for( long int i=0 ; i < numPartToInject ; i++ ) {
		// get initial position and velocity for new particle
		// define values for new particle
		new_p.init();
		new_p.set_q( sp.get_q() );
		new_p.set_m( sp.get_m() );
		new_p.set_sp_index( this->sp_index );
		// define values for new particle
		new_p.set_r( inject_position(i) );
		new_p.set_v( inject_velocity(i) );
		// add particle to particle list
		retList += new_p ;
	}
#endif

	return retList;
}


/**
 * Print content of initial load data to stdout
 */
void InitialLoad::dump(int index) {
	cout << endl;
	cout << "Initial load data dump:" << endl;
	cout << "=======================" << endl;
	cout << "Block index in input file: " << index << endl;
	cout << "Species index: " << sp_index;
	cout << sp.dump2string() << endl;
	cout << "Number of particles to load: " << numPartToInject << endl;

	// dump position and velocity distributions and other data
	dumpPositions("");
	dumpVelocities("");
}

