/*
 * InputFile.cpp
 *
 *  Created on: 2011-08-07
 *      Author: janez
 */

#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <sstream>
#include <string>
#include <cstring>
#include <vector>

#include "InputFile.h"
#include "Vector.h"
#include "InitialLoad.h"
#include "Grid.h"

using namespace std;

InputFile::InputFile() {
	filename = "";
}

InputFile::~InputFile() {

	if( vars.size() > 0 ) {
		vars.clear();
	}
}

/**
 * Read content of input file and store it in blocks.
 */
void InputFile::read() {
	int loop = 1, level;
	long int line_num;
	char tag[MAX_VAR_NAME+1], line[LINE_LEN+1], line2[LINE_LEN+1];
	FILE* fp;

	char *fname = (char *)filename.c_str();

	if( (fp = fopen(fname, "r")) == NULL ) {
		/* cannot open input file for reading */
		cout << "Error reading input file " << fname << "." << endl;
		exit(1);
	}

	/*
	 * read input file into tree of blocks; each section in input file
	 * is presented as one node in tree. Variables inside section are stored in
	 * structure values. Sections inside current sections are stored in list of
	 * blocks.
	 */
	level = 0;
	line_num = 0;
	fgets(line, LINE_LEN, fp);
	line_num++;

	while( loop && !feof(fp) ) {
		/* loop until we find start of first block, this is filename (or project name) */
		fgets(line2, LINE_LEN, fp);
		line_num++;
		tag[0] = '\0';
		sscanf(line2, "%s", tag);

		if( tag[0] == '{' ) {
			/* first read line (line) is name of file/project name */
			sscanf(line, "%s", tag);
			strncpy(file_content.name, tag, MAX_SECTION_NAME);
			file_content.line_num = line_num;
			level++;

			if( read_block( fp, &file_content, &line_num, level ) ) {
				/* error reading input file */
				cout << endl << " Errors in reading file." << endl;
				fclose(fp);
				exit(1);
			}

		} else if( tag[0] == '}' ) {
			/* end of block */
			if( level > 0 ) {
				/* not on start level; lower one level */
				level --;
			} else {
				/* error, to many } */
				cout << "Error in line " << line_num << ": too many }'s." << endl;
				loop = 0;
				fclose(fp);
				exit(1);
			}

		} else {
			/* store current line into var line and do another loop */
			strncpy(line, line2, LINE_LEN+1);
		}
	}

	fclose(fp);

	/* remove comment from next line for checking */
	//print_block( &file_content, 0 );

	// fill array with variables
	struct file_block* file_vars;

	if( (file_vars = this->get_block(NULL, "Variables") ) != NULL ) {
		// we have variables defined in current input file
		struct variable var;

		for( unsigned int i=0 ; i < file_vars->values.size() ; i++) {
			// fill array
			var.name = file_vars->values[i].name;
			sscanf( file_vars->values[i].value, "%lg", &(var.value) );
			vars.push_back(var);
		}
	}

	return;
}


/*
 * Read input file into tree of blocks; each block presents one section from input file.
 * Meaning of each block and values in block (and storing values from file into structures for simulation)
 * will be done later.
 *
 * Here we do not try to understand values and block (this is done later);
 * We just try to read information from input file; we display error if anything is wrong with format of input file.
 *
 * Return: 0= OK, no problems, >0 = troubles; error was written to stdout
 */
int InputFile::read_block( FILE *fp, struct file_block *block, long int *line_num, int level ){
	char line[LINE_LEN+1], line2[LINE_LEN+1], word[MAX_SECTION_NAME+1];
	char *cp, *cp1, *cp2, *cp3, *cp_line2;
	int loop = 1, loadNextLine=1, ret = 0;
	long int start_line_num;
	struct file_block newb;
	struct section_var new_var;


	start_line_num = *line_num;

	while( loop && !feof(fp) ) {
		/* loop until we read all data for species or there is end of file */
		if( loadNextLine ) {
			/* read next line */
			fgets(line, LINE_LEN, fp);
			(*line_num) ++;
		}

		loadNextLine = 1;
		word[0] = '\0';
		sscanf(line, "%s", word);

		if( word[0] == '\0' ) {
			/* empty line; skip it */

		} else if( word[0] == '{' ) {
			/* begin of species block; skip to next line */
			if( start_line_num + 1 == *line_num ) {
				/* another { at beginning of block */
				cout << "Error in line " << *line_num << ": two { in the beginning of the block." << endl;
			} else {
				cout << "Error in line " << *line_num << ": start of block without block name. Maybe empty line before {?" << endl;
			}
			loop = 0;
			ret = 1;

		} else if( word[0] == '}' ) {
			/* end of block; */
			loop = 0;

		} else if( strncmp(word,"//", 2) == 0 ) {
			/* current line is comment; skip it all together */

		} else {
			/* let's see what to do */
			/* read next line and get first word from line */
			fgets(line2, LINE_LEN, fp);
			(*line_num) ++;

			/* find first non-space char in second line */
			for( cp_line2=line2 ; *cp_line2 == ' ' || *cp_line2 == '\t' ; cp_line2++ );

			if( *cp_line2 == '{' ) {
				/* current line is name of new section */
				newb = init_new_block( word, *line_num );

				if( (ret = read_block( fp, &newb, line_num, level+1 )) ) {
					/* there was some error reading current block */
					loop = 0;
				}

				// store block into list of block in current block
				block->blocks.push_back(newb);

			} else {
				/* we have a normal line */
				if( strchr( line, '=' ) != NULL ) {
					/* we have line with "=" in it; value for some variables is defined */
					/* we define 4 pointers:
					 *  cp  - start of variable name
					 *  cp1 - end of variable name
					 *  cp2 - start of variable value
					 *  cp3 - end of variable value
					 */
					for( cp=line ; *cp == ' ' || *cp == '\t' || *cp == '=' ; cp++ ) ;
					for( cp1=cp ; !( *cp1 == ' ' || *cp1 == '\t' || *cp1 == '=' || *cp1 == '\n' ) ; cp1++ ) ;
					for( cp2=cp1 ; *cp2 == ' ' || *cp2 == '\t' || *cp2 == '=' || *cp2 == '\n' ; cp2++ ) ;
					// end of value: ";", //, "new-line"
					for( cp3=cp2 ; !( *cp3 == ';' || (*(cp3-1) == '/' && *cp3 == '/')|| *cp3 == '\n' ) ; cp3++ ) ;

					do {
						cp3--;
					} while(isblank(*cp3) || *cp3 == '/');

					cp3++;

					// version that read only first word after =
					//for( cp3=cp2 ; !( isblank(*cp3) || *cp3 == ';' || *cp3 == '/' || *cp3 == '\n' ) ; cp3++ ) ;

					/* terminate sub-strings */
					*cp1 = *cp3 = '\0';

					// initialize the variable
					init_new_variable( &new_var );
					/* copy to variable; because we copy 1 char less than size of strings, string is always NULL terminated */
					strncpy( new_var.name, cp, MIN(cp1-cp+1, MAX_VAR_NAME) );
					strncpy( new_var.value, cp2, MIN(cp3-cp2+1, MAX_VAR_VALUE) );
					// add it to list of values
					block->values.push_back(new_var);

				} else {
					/* we assume that something is defined; find first "word" in line and use it as variable */
					/* we define 2 pointers:
					 *  cp  - start of variable name
					 *  cp1 - end of variable name
					 */
					for( cp=line ; *cp == ' ' || *cp == '\t' || *cp == '=' ; cp++ ) ;
					for( cp1=cp ; !( isblank(*cp1) || *cp1 == ';' || *cp1 == '/' || *cp1 == '\n' ) ; cp1++ ) ;

					if( *cp != '\n' && *cp != '\0' ) {
						/* copy to variable; because we copy 1 char less than size of strings, string is always NULL terminated */
						/* terminate sub-strings */
						*cp1 = '\0';

						// initialize the variable
						init_new_variable( &new_var );
						/* copy to variable; because we copy 1 char less than size of strings, string is always NULL terminated */
						strncpy( new_var.name, cp, MIN(cp1-cp+1, MAX_VAR_NAME) );
						strcpy( new_var.value, "" );
						// add it to list of values
						block->values.push_back(new_var);
					}
				}

				/* copy next line into current line for next loop and
				 * mark not to read new line from file */
				strncpy( line, line2, LINE_LEN );
				loadNextLine = 0;
			}
		}
	}

	return ret;
}


/**
 * Initialize tree block node;
 */
struct file_block InputFile::init_new_block( char* name, int line_num ) {
	struct file_block newb;


	// make block empty
	delete_block( &newb );
	// copy name of block
	strncpy( newb.name, name, MAX_SECTION_NAME);
	/* store current line number - for displaying errors */
	newb.line_num = line_num;

	return newb;
}


/*
 * Delete given block; freeing all memory
 */
void InputFile::delete_block( struct file_block *block ) {
	unsigned int i;

	if( block == NULL ) {
		/* given pointer to block is already NULL; return */
		return;
	}

	if( block->blocks.size() > 0 ) {
		/* clear all defined blocks */
		for( i=0 ; i < block->blocks.size() ; i++ ) {
			delete_block( &(block->blocks[i]) );
		}

		block->blocks.clear();
	}

	if( block->values.size() > 0 ) {
		/* clear all defined values */
		block->values.clear();
	}

	/* delete content of name */
	for( i=0 ; i < MAX_SECTION_NAME+1 ; i++ ) {
		block->name[i] = '\0';
	}
}


/*
 * Initialize given variable.
 */
void InputFile::init_new_variable( struct section_var *new_var ) {
	new_var->name[0] = '\0';
	new_var->value[0] = '\0';
}


/*
 * Print data of given block. Level defined number of spaces to indent printout.
 */
void InputFile::print_block( struct file_block *block, int level ){
	char offset[2*MAX_DEPTH];
	unsigned int i=0;
	int j;


	for( j=0 ; j < 2*level ; j++ ) {
		offset[j] = ' ';
	}

	offset[2*level] = '\0';

	/* name of block */
	printf("\n%sBlock: %s", offset, block->name);

	if( block->values.size() > 0 ) {
		printf("\n%sVariables/definitions:", offset);

		for( i=0 ; i < block->values.size() ; i++ ) {
			/* first print values */
			printf("\n%s - %s", offset, block->values[i].name );

			if( block->values[i].value[0] != '\0' ) {
				/* print also value ot variable */
				printf(": %s", block->values[i].value );
			}
		}
	} else {
		/* no definitions */
		printf("\n%sNo variables or definitions.", offset );
	}

	for( i=0 ; i < block->blocks.size() ; i++ ) {
		/* then print blocks under current block */
		print_block( &(block->blocks[i]), level+1 );
	}

	printf("\n");
}

/**
 * Return pointer to block with given name of NULLif block does not not exist.
 */
struct file_block* InputFile::get_block(struct file_block* in_fb, std::string name) {
	struct file_block* fbret, *fb;
	unsigned int i=0;

	fbret = NULL;

	if( in_fb == NULL ) {
		// empty block given; use file_content
		fb = &file_content;
	} else {
		// use given block
		fb = in_fb;
	}

	for( i=0 ; i < fb->blocks.size() && fbret == NULL ; i++ ) {
		// check all blocks
		if( strcmp(fb->blocks[i].name, name.c_str() ) == 0 ) {
			// got block with given name;
			fbret = &fb->blocks[i];
		}
	}

	return fbret;
}



/*
 * Store value of given variable in given third parameter;
 * Return 1 on success or 0 on error (variable name does not exist)
 */
int InputFile::get_var_value(char *s_val, double *d_val){
	int ret = 1, loop = 1;
	unsigned i;

	for( i=0 ; i < vars.size() && loop; i++) {
		/* loop through all variables till we find one or we checked all of them */
		if( strcmp(s_val, vars[i].name.c_str()) == 0 ) {
			/* OK, variable found */
			*d_val = vars[i].value;
			loop = 0;
		}
	}

	if( loop == 1 ) {
		*d_val = 0;
		ret = 0;
	}

	return ret;
}


/*
 * Find variable/definition name (name) in given block (block) and store it into dest.
 * Return 1 on success or 0 on error (value name does not exist)
 */
int InputFile::get_char_value( std::string& dest, std::string name, struct file_block *bp ){
	int ret = 0, loop = 1;
	unsigned int i = 0;

	dest = "";

	for( i=0 ; i < bp->values.size() && loop; i++) {
		/* loop through all values till we find one or we checked all of them */
		if( strcmp( bp->values[i].name, name.c_str() ) == 0 ) {
			/* name was found; store it into dest and return 1 */
			dest = bp->values[i].value;

#if 0
			len = strlen(bp->values[i]->value);
			strncpy(dest, bp->values[i]->value, len);
			*(dest+len) = '\0';				/* destination string has to be null-terminated */
#endif

			ret = 1;
			loop = 0;
		}
	}

	return ret;
}


/*
 * Find double value with name (name) in given block (block) and store it into dest.
 * If value in current block is variable, check if variable exists and store that value.
 * Return 1 on success or 0 on error (value name does not exist; return value is undefined)
 */
int InputFile::get_double_value( double *dest, std::string name, struct file_block *bp ){
	int ret = 0, loop = 1;
//	int len = name.length();
	unsigned i;
	char *endp;
	double val;

	*dest = 0.;

	for( i=0 ; i < bp->values.size() && loop; i++) {
		/* loop through all values till we find one or we checked all of them */
		if( strcmp( bp->values[i].name, name.c_str() ) == 0 ) {
			/* value was found */
			endp = NULL;		/* to notice difference after function call */

			val = strtod( bp->values[i].value, &endp );

			if( endp == bp->values[i].value ) {
				/* value is not a number; check variables */
				if( vars.size() > 0 ) {
					/* variables are given; check them for value */
					ret = get_var_value( bp->values[i].value, &val );
					*dest = val;
					loop = 0;
				} else {
					/* no value; return value is undefined */
					loop = 0;
					ret = 0;
				}
			} else {
				/* store value and return */
				*dest = val;
				ret = 1;
				loop = 0;
			}
		}
	}

	return ret;
}


/*
 * Find integer value with name (name) in given block (block) and store it into dest.
 * If value in current block is variable, check if variable exists and store that value.
 * Return 1 on success or 0 on error (value name does not exist; return value is undefined)
 */
int InputFile::get_int_value( int *dest, std::string name, struct file_block *bp ){
	int ret = 0;
	double val;


	if( get_double_value( &val, name, bp ) ) {
		/* we got double value; convert it to integer and return */
		*dest = (int) val;
		ret = 1;
	}

	return ret;
}


/*
 * Find long integer value with name (name) in given block (block) and store it into dest.
 * If value in current block is variable, check if variable exists and store that value.
 * Return 1 on success or 0 on error (value name does not exist; return value is undefined)
 */
int InputFile::get_long_value( long int *dest, std::string name, struct file_block *bp ){
	int ret = 0, loop = 1;
//	int len = name.length();
	unsigned i;
	long int  val;
	stringstream str;

	*dest = 0;

	for( i=0 ; i < bp->values.size() && loop; i++) {
		/* loop through all values till we find one or we checked all of them */
		if( strcmp( bp->values[i].name, name.c_str() ) == 0 ) {
			/* value was found */
			str << bp->values[i].value;
			str >> val;
			/* store value and return */
			*dest = val;
			ret = 1;
			loop = 0;
		}
	}

	return ret;

}


/**
 * Return value of input file test mode: true/false
 */
bool InputFile::getInputFileTestMode() {
	bool ret = false;
	std::string line;

	if( get_char_value(line, "test_mode", &file_content) ) {
		// yes, ignore current diagnostics
		ret = true;
	}

	return ret;
}

/**
 * Define and fill species from input file
 */
std::vector<Species> InputFile::getSpecies( std::string block_name ) {
	std::vector<Species> sp_list;
	Species sp;
	struct file_block* fb = NULL;
	int* block_pos = new int[file_content.blocks.size()];
	int sp_def_name, sp_def_q, sp_def_m;
	unsigned int i, numSpecies=0;
	double dval1, dval2;
	char cval[MAX_VAR_NAME+1];
	std::string sp_name;


	for( i=0 ; i < file_content.blocks.size() ; i++ ) {
		// parse all blocks and store index of species blocks
		if( strcmp(file_content.blocks[i].name, block_name.c_str() ) == 0 ) {
			// current block is species;
			block_pos[numSpecies] = i;
			numSpecies ++;
		}
	}

	if( numSpecies == 0 ) {
		// no species; return
		return sp_list;
	}


	// now we have index of species blocks; allocate array

	for( i=0 ; i < numSpecies ; i++ ) {
		// fill data for species
		// allocate new species
		sp.init();
		// get file block with species data
		fb = &(file_content.blocks[block_pos[i]]);

		sp_def_name = sp_def_q = sp_def_m = 0;		/* which part of species is defined */
		/* get name of species */
		sp_def_name = get_char_value( sp_name, "name", fb );
		/* get species properties; read from variables if necessary */
		sp_def_q = get_double_value( &dval1, "q", fb );
		sp_def_m = get_double_value( &dval2, "m", fb );

		if( !(sp_def_name && sp_def_q && sp_def_m) ) {
			/* one of important parameters was not defined */
			cout << "Error in line " << fb->line_num << ": not enough data for species named " << cval << "." << endl;
		} else {
			// add values
			sp.set_name(sp_name);
			sp.set_q(dval1);
			sp.set_m(dval2);
			sp_list.push_back(sp);

			// just for check -- dump species values
			// sp.dump();
		}
	}

	return sp_list;
}


/**
 * Define and fill data for plasma device from input file
 */
SpatialRegion InputFile::getSpatialRegion( string block_name ) {
	SpatialRegion sr;
	Grid grid;
	Vector3 r0, r1;
	struct file_block* fb = NULL;
	struct file_block* fb1 = NULL;
	double dval;
	long int lival;
	std::string comp_method;


	// get device block
	fb = get_block( NULL, block_name );

	if( fb != NULL ) {
		// fill values from given block
		sr.init();

//		print_block( fb, 1);

		if( get_char_value( comp_method, "compute_method", fb ) == 1 ) {
			// try to define compute method...
			if( comp_method == "green" ) {
				sr.set_compute_method( Compute::COMPUTE_METHOD_GREEN );
			}

		}

		// try to read info from spatial region
		get_double_value( &dval, "dt", fb );
		sr.set_dt( dval );
		get_long_value( &lival, "ns", fb );
		sr.set_maxStep( lival );
		get_double_value( &dval, "area", fb );
		sr.set_area( dval );

		if( get_double_value( &dval, "p2c", fb ) ) {
			// value for ratio between computer and real particles is defined;
			sr.set_p2c( dval );
		} else {
			// use 1 as ratio between real an computer particles
			sr.set_p2c( 1. );
		}

		//
		// temporary; read values for border potentials (V)
		//
		get_double_value( &dval, "V0", fb );
		sr.set_V0( dval );
		get_double_value( &dval, "V1", fb );
		sr.set_V1( dval );
		//
		// end of temporary part
		//

		if( (fb1 = get_block(fb, "Grid")) != NULL ) {
			// we have a grid definition
			r0.set(0.,0.,0.);
			r1.set(0.,0.,0.);
			grid.init();
			string coords[3] = { "x", "y", "z" };

			for( int i=0 ; i < 3 ; i++ ) {
				// check all vales and add them to vectors (for defining grid)
				if( get_double_value( &dval, coords[i] + "0", fb1 ) ){
					// define min. x for grid
					r0.set( i+1, dval );
				}

				if( get_double_value( &dval, coords[i] + "1", fb1 ) ){
					// define max. x for grid
					r1.set( i+1, dval );
				}

				if( get_long_value( &lival, "n" + coords[i], fb1 ) ) {
					// number of cells in current direction is defined
					// save number of points on grid (cells + 1)
					grid.set_n( i, lival+1 );
				}
			}

			if( get_long_value( &lival, "ng", fb1 ) ) {
				// sometimes, number of cells in x-direction is defined as "ng";
				// save number of points on grid (cells + 1)
				grid.set_nx( lival+1 );
			}

			// define grid limits from 2 vectors
			grid.set_r0( r0 );
			grid.set_r1( r1 );
			// define grid
			sr.set_grid( grid );
		}
	}

	return sr;
}


/**
 * Define and fill sources from input file
 */
std::vector<Source> InputFile::getSources( std::string block_name, std::vector<Species> &species ) {
	std::vector<Source> slist;
	Source s;
	struct file_block* fb = NULL;
	int* block_pos = new int[file_content.blocks.size()];
	size_t i, numSources=0;
	int ival;
	long int lival;
	double dval1;
	Vector3 r, v;
	string line;
	std::vector<string> errors;

	for( i=0 ; i < file_content.blocks.size() ; i++ ) {
		// parse all blocks and store index of source blocks
		if( strcmp(file_content.blocks[i].name, block_name.c_str() ) == 0 ) {
			// current block is source;
			block_pos[numSources] = i;
			numSources ++;
		}
	}

	if( numSources == 0 ) {
		// no sources; return
		return slist;
	}

	// now we have index of source blocks; allocate array

	for( i=0 ; i < numSources ; i++ ) {
		// fill data for sources
		s.init();
		// get file block with source data
		fb = &(file_content.blocks[block_pos[i]]);

		// first check if current source is disabled
		//
		if( get_char_value(line, "disabled", fb) ) {
			// yes, ignore current source
			continue;
		}

		/* definition of single particle; store into calc_params */
		/* get name of species */
		if( get_char_value(line, "species", fb) ) {
			/* species is defined; check if it exists */
			if( (ival = isSpeciesDefined( line, species )) != -1 ) {
				/* given species name exists; store index into source */
				// variable ival is index in species array; define species
				s.set_sp( species[ival], ival );
			}
		}

		if( get_double_value( &dval1, "dt", fb ) ) {
			// get time between each injection of particles
			s.set_dt( dval1 );
		}

		if( get_long_value( &lival, "di", fb ) ) {
			// get number of steps between each injection of particles
			s.set_dStep( lival );
		}

		if( get_double_value( &dval1, "Tpar", fb ) ) {
			// get injection parallel temperature (Tx)
			s.set_Tpar( dval1 );
		}

		if( get_double_value( &dval1, "Tperp", fb ) ) {
			// get injection perpendicular temperature (Ty;Tz)
			s.set_Tperp( dval1 );
		}

		// load data
		readParticlePositions( &s, fb );
		readParticleVelocities( &s, fb );
		// check for errors (if all minimal set of data is defined)
		errors = s.definedDataOK(species);

		if( errors.size() > 0 ) {
			// there are some errors here; display returned message
			cout << "Error in line " << fb->line_num << ": source definition errors: " << endl;

			for( i=0 ; i < errors.size() ; i++ ) {
				// display all errors
				cout << "     - " << errors[i] << endl;
			}

			exit(1);
		}

		slist.push_back( s );
	}

	return slist;
}


/**
 * Define and fill sources from input file
 */
Tree InputFile::getTree( std::string block_name ) {
	Tree t;
	struct file_block* fb = NULL;
	int ival;
	double dval;
	string line;


	// get device block
	fb = get_block( NULL, block_name );

	if( fb != NULL ) {
		// fill values from given block
		t.init();

//		print_block( fb, 1);

		// try to read info from spatial region
		get_int_value( &ival, "N0", fb );
		t.set_maxParticlesInNode( ival );
		get_double_value( &dval, "nodeLengthRatio", fb );
		t.set_nodeLengthRatio( dval );
		get_double_value( &dval, "p2cRatio", fb );
		t.set_p2c( dval );

		if( get_char_value(line, "use_direct_simulation", fb) ) {
			// check if we have to append to file
			t.treecode_disable();
		}
	}

	return t;
}


/**
 * Store initial load into object for later use.
 */
vector<InitialLoad> InputFile::getInitLoad( string block_name, std::vector<Species> &species ) {
	InitialLoad load;
	long int ival;
	vector<InitialLoad> initload_list;
	struct file_block* fb = NULL, *fb1;
	unsigned int iblock;
	size_t i;
	string line;
	std::vector<std::string> errors;

	// get device block
	fb = get_block( NULL, block_name );

	if( fb != NULL ) {
		// fill values from given block
		for( iblock = 0 ; iblock < fb->blocks.size() ; iblock++ ) {
			// parse all block in current init block
			fb1 = &(fb->blocks[iblock]);

			if( strcmp(fb->blocks[iblock].name, "Particles") == 0 ) {
				/* first get number of particles (n); if it is more than 1,
				 * we expect area definition, otherwise it is definition of a
				 * single particle */
				load.init();

				// first check if current initial load is disabled
				//
				if( get_char_value(line, "disabled", fb1) ) {
					// yes, ignore current initial load
					continue;
				}

				/* get name of species */
				if( get_char_value(line, "species", fb1) ) {
					/* species is defined; check if it exists */
					if( (ival = isSpeciesDefined( line, species )) != -1 ) {
						/* given species name exists; store index into source */
						// variable ival is index in species array; define species
						load.set_sp( species[ival], ival );
					}
				}

				// load data
				readParticlePositions( &load, fb1 );
				readParticleVelocities( &load, fb1 );
				// check for errors (if all minimal set of data is defined)
				errors = load.definedDataOK(species);

				if( errors.size() > 0 ) {
					// there are some errors here; display returned message
					cout << "Error in line " << fb->line_num << ": initial load definition errors: " << endl;

					for( i=0 ; i < errors.size() ; i++ ) {
						// display all errors
						cout << "     - " << errors[i] << endl;
					}

					exit(1);
				}

				// add to initial load
				initload_list.push_back(load);

				load.dump(0);
			}

		}

	}

	return initload_list;
}

/**
 * Check if species with given name is defined and return index of species.
 * If species is not defined, return -1
 * @return 0 or more: index of species in list of species, -1: not defined
 */
int InputFile::isSpeciesDefined(std::string name, std::vector<Species> &species) {
	int pos = -1;
	unsigned i;


	for( i=0 ; i < species.size() && pos == -1; i++ ) {
		/* check all defined species */
		if( species[i].get_name() == name ) {
			/* OK, species found; return position */
			pos = i;
		}
	}

	return pos;
}


/**
 * Read data from given block with given names and store it into vector.
 *
 */
Vector3 InputFile::get_vector_coords( std::string x, std::string y, std::string z, struct file_block *fb ) {
	double tmp;
	Vector3 vec;

	// initialize vector
	vec.set(0., 0., 0.);

	// get all three values
	get_double_value( &tmp, x, fb );
	vec.set_e1(tmp);
	get_double_value( &tmp, y, fb );
	vec.set_e2(tmp);
	get_double_value( &tmp, z, fb );
	vec.set_e3(tmp);

	return vec;
}


/**
 * Get fields from input file.
 */
std::vector<Field> InputFile::getFields( std::string block_name ) {
	Field field;
	double dval;
	vector<Field> field_list;
	struct file_block* fb = NULL, *fb1;
	unsigned int iblock;
	string line;



	// get device block
	fb = get_block( NULL, block_name );

	if( fb != NULL ) {
		// fill values from given block
		for( iblock = 0 ; iblock < fb->blocks.size() ; iblock++ ) {
			// parse all block in current init block
			fb1 = &(fb->blocks[iblock]);

			if( strcmp(fb->blocks[iblock].name, "Field") == 0 ) {
				/* first get number of particles (n); if it is more than 1,
				 * we expect area definition, otherwise it is definition of a
				 * single particle */
				field.init();

				// first check if current field is disabled
				//
				if( get_char_value(line, "disabled", fb1) ) {
					// yes, ignore current field
					continue;
				}

				if( get_char_value(line, "type", fb1) ) {
					// we got field type
					if( line == "E" || line == "electric" ) {
						// el. field
						field.setType(Field::FIELD_TYPE_E);

					} else if( line == "B" || line == "magnetic" ) {
						// mag. field
						field.setType(Field::FIELD_TYPE_B);

					} else {
						// wrong field type
						cout << "Error in line " << fb1->line_num << ": field type not correct, should be B or E." << endl;
						exit(1);
					}

				} else {
					// field type definition is missing
					cout << "Error in line " << fb1->line_num << ": field type not defined." << endl;
					exit(1);
				}

				if( get_double_value( &dval, "value", fb1 ) ) {
					// we read value of field
					field.setAbs_value(dval);
				} else {
					// no value; error
					cout << "Error in line " << fb1->line_num << ": field value not defined." << endl;
					exit(1);
				}

				if( get_double_value( &dval, "angle_xz", fb1 ) ) {
					// we read angle of field according to X-Z plane
					field.setAngle_yz(dval);
				}

				if( get_double_value( &dval, "angle_xy", fb1 ) ) {
					// we read angle of field according to X-Y plane
					field.setAngle_xy(dval);
				}

				// add to initial fields
				field_list.push_back(field);
			}

		}

	}

	return field_list;
}


/**
 * Define and fill diagnostics data from input file.
 */
vector<Diagnostics> InputFile::getDiagnostics( std::string block_name, std::vector<Species> &species ) {
	vector<Diagnostics> diag_list;
	Diagnostics cDiag;
	long int ival;
	struct file_block* fb = NULL, *fb1;
	unsigned int iblock;
	string line;
	string output_dir = "";

	// get device block
	fb = get_block( NULL, block_name );

	if( fb != NULL ) {
		// fill values from given block
		// read default settings for all diagnostics sub-blocks (output dir, etc.)
		if( get_char_value(line, "directory", fb) ) {
			// output directory is defined;
			output_dir = line;
		}

		//
		// now read all diagnostics blocks and use before read values as default values
		//
		for( iblock = 0 ; iblock < fb->blocks.size() ; iblock++ ) {
			// parse all block in diagnostics block
			fb1 = &(fb->blocks[iblock]);
			cDiag.Init();
			cDiag.set_output_directory( output_dir );

			// first check if current diagnostics is disabled
			//
			if( get_char_value(line, "disabled", fb1) ) {
				// yes, ignore current diagnostics
				continue;
			}

			//
			// read values from current block;
			//
			if( get_char_value(line, "file", fb1) ) {
				// output file
				cDiag.set_file( line );
			} else {
				// output file not defined; error
				cout << "Error in line " << fb1->line_num << ": output file not defined." << endl;
				exit(1);
			}

			if( get_char_value(line, "species", fb1) ) {
				// species are defined
				bool done = false;
				size_t pos = 0;
				string spec_name = "";

				line = stringTrim( line );

				while( !done ) {
					// parse whole string
					if( (pos = line.find(" ")) != std::string::npos ) {
						// we still have list of species; get end of species name
						// get species name
						spec_name = line.substr(0, pos);
						// prepare line for next loop
						line.erase(0, pos);
						line = stringTrim(line);
					} else {
						// whole line is species name
						spec_name = line;
						done = true;
					}

					// check if species is defined
					spec_name = stringTrim( spec_name );

					if( (ival = isSpeciesDefined( spec_name, species )) != -1 ) {
						// OK, given species is defined; save it to list of species in diagnostics
						cDiag.species_add(ival);

					} else {
						// given species name does not exist
						cout << "Error in line " << fb1->line_num << ": species name '" << spec_name << "' unknown or not defined." << endl;
						exit(1);
					}
				}
			}

			if( strcmp(fb->blocks[iblock].name, "PhaseSpace") == 0 ) {
				// phase-space output
				cDiag.set_diagnostics_type( Diagnostics::DIAGNOSTICS_PHASE_SPACE );

			} else if( strcmp(fb->blocks[iblock].name, "PotentialProfile") == 0 ) {
				// potential profile output
				cDiag.set_diagnostics_type( Diagnostics::DIAGNOSTICS_POTENTIAL_PROFILE );

			} else if( strcmp(fb->blocks[iblock].name, "NumberOfParticles") == 0 ) {
				// output of number of particles
				cDiag.set_diagnostics_type( Diagnostics::DIAGNOSTICS_NUMBER_OF_PARTICLES );

			} else if( strcmp(fb->blocks[iblock].name, "MemoryUsage") == 0 ) {
				// output of number of particles
				cDiag.set_diagnostics_type( Diagnostics::DIAGNOSTICS_MEMORY_USAGE );

			} else if( strcmp(fb->blocks[iblock].name, "Density") == 0 ) {
				// output of number of particles
				cDiag.set_diagnostics_type( Diagnostics::DIAGNOSTICS_DENSITY );

			} else if( strcmp(fb->blocks[iblock].name, "Current") == 0 ) {
				// output of current
				cDiag.set_diagnostics_type( Diagnostics::DIAGNOSTICS_CURRENT );

			}

			if( get_char_value(line, "append", fb1) ) {
				// check if we have to append to file
				cDiag.set_write_append( true );
			}

			// read definition of write points
			diagnosticsWritePoints( &cDiag, fb1 );

			// add to list of diagnostics
			diag_list.push_back(cDiag);
		}

	}

	return diag_list;
}


/**
 * Read write points from given block and store them into given diagnostics object.
 * Write points are points (in time or computational steps) when output data (potential, points coordinates, ....)
 * should be written to file, defined inside current block.
 */
void InputFile::diagnosticsWritePoints( Diagnostics* diag, struct file_block* fb ) {
	string line;
	long int lival = 0, lival1, lival2;
	double dval, dval1, dval2;
	std::vector<long> ltmp;
	std::vector<double> dtmp;


	// no writing will be done
	diag->set_number_of_files( 0 );
	diag->set_write_data_by_step( 0, 0, 0 );
	diag->set_write_data_by_time( 0.0, 0.0, 0.0 );

	if( get_long_value( &lival, "n", fb ) ) {
		// only one step is defined; use that
		// because we have only one step, we write only one file
		diag->set_number_of_files( 1 );
		diag->set_write_data_by_step( lival, 0, 0 );
		return;
	}

	if( get_double_value( &dval, "t", fb ) ) {
		// only one step is defined; use that
		// because we have only one step, we write only one file
		diag->set_number_of_files( 1 );
		diag->set_write_data_by_time( dval, 0, 0 );
		return;
	}

	if( get_long_value( &lival, "n_step", fb ) ) {
		// there is range defined on basis of computational steps
		diag->set_number_of_files( 0 );
		diag->set_write_data_by_step( 0, lival, 0 );

		if( get_long_value( &lival, "n_start", fb ) ) {
			// start step is defined
			diag->set_write_data_by_step_start( lival );
		}

		if( get_long_value( &lival, "n_end", fb ) ) {
			// end step is defined
			diag->set_write_data_by_step_end( lival );
		}

		// now quick check
		ltmp = diag->get_write_data_by_step();

		if( ltmp[2] != 0 && ltmp[2] < ltmp[0] ) {
			// end step is smaller than first one; swap them
			diag->set_write_data_by_step_start( ltmp[2] );
			diag->set_write_data_by_step_end( ltmp[0] );
		}

		return;
	}

	if( get_double_value( &dval, "t_step", fb ) ) {
		// there is range defined on basis of computational time
		diag->set_number_of_files( 0 );
		diag->set_write_data_by_time( 0, dval, 0 );

		if( get_double_value( &dval, "t_start", fb ) ) {
			// start time is defined
			diag->set_write_data_by_time_start( dval );
		}

		if( get_double_value( &dval, "t_end", fb ) ) {
			// end time is defined
			diag->set_write_data_by_time_end( dval );
		}

		// now quick check
		dtmp = diag->get_write_data_by_time();

		if( dtmp[2] != 0 && dtmp[2] < dtmp[0] ) {
			// end step is smaller than first one; swap them
			diag->set_write_data_by_time_start( dtmp[2] );
			diag->set_write_data_by_time_end( dtmp[0] );
		}

		return;
	}

	if( get_long_value( &lival, "files", fb ) ) {
		// number of files is defined
		diag->set_number_of_files( lival );
		lival1 = lival2 = 0;
		dval1 = dval2 = 0.0;

		int def_n_start = get_long_value( &lival1, "n_start", fb );
		int def_n_end = get_long_value( &lival2, "n_end", fb );
		int def_t_start = get_double_value( &dval1, "t_start", fb );
		int def_t_end = get_double_value( &dval2, "t_end", fb );

		if( def_n_start || def_n_end ) {
			// one of values (start or end) is defined; use step for write points
			// set as 0 to use "files"
			diag->set_write_data_by( TYPE_DELTA_STEP );
			diag->set_write_data_by_step( lival1, 0, lival2 );
			return;

		} else if( def_t_start || def_t_end ){
			// one of values (start or end) is defined; use time for write points
			// set as 0 to use "files"
			diag->set_write_data_by( TYPE_DELTA_TIME );
			diag->set_write_data_by_time( dval1, 0.0, dval2 );
			return;

		} else {
			// no start of end values were defined; use computational step
			// create given number of files from start to end of simulation
			diag->set_write_data_by( TYPE_DELTA_STEP );
			diag->set_write_data_by_step( 0, 0, 0 );
			return;
		}
	}

	// if we come to here, some data is missing.
	cout << "Error in line " << fb->line_num << ": write points (time/steps when to write output to files) are not defined. Use 'files' to define number of files to write." << endl;
	exit(1);

	return;
}

/**
 * Read particle distribution from given file block.
 */
void InputFile::readParticlePositions( Source* src, struct file_block* fb ) {
	Vector3 r, r_min, r_max;
	string line;
	long int lival = 0;
	double dval0, dval1;
	bool pos_min = false, pos_max = false;


	r = get_vector_coords( "x", "y", "z", fb );
	get_long_value( &lival, "n", fb );
	// store number of particles
	src->set_numberToInject( lival );

	if( get_double_value( &dval0, "x", fb ) ) {
		// there is definition of single injection position (can have 1 or more particles to inject)
		src->set_position_distribution(POSITION_DISTRIBUTION_NONE);
		src->set_r( r );
		src->set_r_min(r);
		src->set_r_max(r);

	} else {
		// check for definition of area
		// check for injection range + distribution
		// get positions for range (limits for distribution)
		pos_min = get_double_value( &dval0, "x_min", fb );
		pos_max = get_double_value( &dval1, "x_max", fb );

		if( pos_min && pos_max) {
			// OK, both are defined; check the distribution
			if( !get_char_value(line, "x_positions", fb) ) {
				// position distribution not defined; display error
				cout << "Error in line " << fb->line_num << ": particle position distribution not defined." << endl;
				exit(1);
			} else {
				// save range limits and check if given distribution is OK
				r_min = r;
				r_min.set_e1( dval0 );
				src->set_r_min(r_min);
				r_max = r;
				r_max.set_e1( dval1 );
				src->set_r_max(r_max);

				if( line == "UniformDistribution" ) {
					src->set_position_distribution( POSITION_DISTRIBUTION_UNIFORM ) ;
				} else if( line == "RandomDistribution" ) {
					src->set_position_distribution( POSITION_DISTRIBUTION_RANDOM ) ;
				} else if( line == "UniformNoBordersDistribution" ) {
					src->set_position_distribution( POSITION_DISTRIBUTION_UNIFORM_NO_BORDERS ) ;
				} else if( line == "UniformNoLeftBorderDistribution" ) {
					src->set_position_distribution( POSITION_DISTRIBUTION_UNIFORM_NO_BORDER_LEFT ) ;
				} else if( line == "UniformNoRightBorderDistribution" ) {
					src->set_position_distribution( POSITION_DISTRIBUTION_UNIFORM_NO_BORDER_RIGHT ) ;
				} else {
					// unknown position distribution; display error and quit
					cout << "Error in line " << fb->line_num << ": particle position distribution (" << line << ") not known." << endl;
					exit(1);
				}
			}

		} else if( !pos_min ) {
			// start position is missing
			cout << "Error in line " << fb->line_num << ": start position for particle distribution range not defined." << endl;
			exit(1);

		} else {
			// end position is missing
			cout << "Error in line " << fb->line_num << ": end position for particle distribution range not defined." << endl;
			exit(1);
		}
	}

}


/**
 * Read velocity for particles for injection (source) from current block in case of multiple particles.
 */
void InputFile::readParticleVelocities( Source* src, struct file_block* fb ){
	Vector3 v, v_min, v_max;
	string line, coords[3] = { "vx", "vy", "vz" };
	double dval0, dval1;

	//
	// read data for particle velocities
	//
	if( !get_char_value(line, "velocities", fb) ) {
		// no velocities keyword -- no distribution (single velocity)
		line = "NoDistribution";
	}

	if( line == "NoDistribution" || line == "Single" ) {
		// set velocity and velocity distribution type
		src->set_velocity_distribution( VELOCITY_DISTRIBUTION_NONE ) ;

		if( get_double_value( &dval0, "vx", fb ) ) {
			// velocity vx defined; all OK
			src->set_v( get_vector_coords( "vx", "vy", "vz", fb ) );

		} else {
			// velocity in x not defined (error)
			cout << "Error in line " << fb->line_num << ": velocity vx not defined." << endl;
			exit(1);
		}


	} else if( line == "RandomDistribution" ) {
		// random velocities
		src->set_velocity_distribution( VELOCITY_DISTRIBUTION_RANDOM ) ;
		// now we read all data for velocities
		v = get_vector_coords( "vx", "vy", "vz", fb );
		v_min = v;
		v_max = v;

		for( int i=0 ; i < 3 ; i++ ) {
			// read velocities for all three coordinates
			if( get_double_value( &dval0, coords[i] + "_min", fb ) ) {
				// set min value
				v_min.set( i+1, dval0 );
			}

			if( get_double_value( &dval1, coords[i] + "_max", fb ) ) {
				// set max value
				v_max.set( i+1, dval1 );
			}
		}

		// set min and max velocities
		src->set_v_min( v_min );
		src->set_v_max( v_max );

	} else if( line == "MaxwellianDistribution" ) {
		// velocities according to Maxwellian
		src->set_velocity_distribution( VELOCITY_DISTRIBUTION_MAXWELLIAN ) ;
		// now we read all data for velocities
		src->set_v( get_vector_coords( "vx", "vy", "vz", fb ) );
		// read also temperatures
		src->set_Tpar( 0. ) ;
		src->set_Tperp( 0. ) ;

		if( get_double_value( &dval1, "Tpar", fb ) ) {
			// got it
			src->set_Tpar( dval1 );
		}

		if( get_double_value( &dval1, "Tperp", fb ) ) {
			// got it
			src->set_Tperp( dval1 );
		}

	} else if( line == "RandomDistributionCold" ) {
		// random velocities
		src->set_velocity_distribution( VELOCITY_DISTRIBUTION_RANDOM_COLD ) ;
		// now we read all data for velocities
		src->set_v( get_vector_coords( "vx", "vy", "vz", fb ));

	} else {
		// unknown distribution
		cout << "Error in line " << fb->line_num << ": unknown velocity distribution '" << line << "'." << endl;
		exit(1);
	}

}



std::string InputFile::stringTrim( const std::string& pString, const string& pWhitespace ) {
	const size_t beginStr = pString.find_first_not_of(pWhitespace);

	if (beginStr == std::string::npos) {
		// no content
		return "";
	}

	const size_t endStr = pString.find_last_not_of(pWhitespace);
	const size_t range = endStr - beginStr + 1;

	return pString.substr(beginStr, range);
}
