/*
 * InputFile.h
 *
 *  Created on: 2011-08-07
 *      Author: janez
 */

#ifndef INPUTFILE_H_
#define INPUTFILE_H_

#include "peg_tc.h"

#include <string>
#include <ctype.h>
#include <vector>

#include "SpatialRegion.h"
#include "Species.h"
#include "Particle.h"
#include "InitialLoad.h"
#include "Source.h"
#include "Tree.h"
#include "Compute.h"
#include "Field.h"
#include "Diagnostics.h"

class Diagnostics;

#define MAX_VAR_NAME		64
#define MAX_VAR_VALUE		64
#define MAX_SECTION_NAME	64
#define MAX_VARIABLES		1000
#define MAX_DEPTH			100		/* max depth of sections in input file */
#define LINE_LEN			1024
#define STRUCT_ALLOC_STEP	10		/* step for allocating structures for tags and variables */

struct variable {
	std::string name;
	double value;
};

struct section_var {
	char name[MAX_VAR_NAME+1];
	char value[MAX_VAR_VALUE+1];
};

struct file_block {
	char name[MAX_VAR_NAME+1];		/* name of section or input file in top section */
	long int line_num;
	std::vector<struct file_block> blocks;
	std::vector<struct section_var> values;
};


class InputFile {
	std::string filename;
	std::vector<struct variable> vars;
	struct file_block file_content;

	int read_block( FILE *fp, struct file_block *root, long int *line_num, int level );
	struct file_block init_new_block( char* name, int line_num );
	void delete_block( struct file_block *block );
	void print_block( struct file_block *block, int level );
	struct file_block *add_to_blocklist( struct file_block *block, char *name );
	int get_var_value(char *s_val, double *d_val);
	int get_char_value( std::string& dest, std::string name, struct file_block *bp );
	int get_double_value( double *dest, std::string name, struct file_block *bp );
	int get_int_value( int *dest, std::string name, struct file_block *bp );
	int get_long_value( long int *dest, std::string name, struct file_block *bp );
	void init_new_variable( struct section_var *new_var );
	int isSpeciesDefined(std::string name, std::vector<Species> &species);
	Vector3 get_vector_coords( std::string x, std::string y, std::string z, struct file_block *fb );
	void diagnosticsWritePoints( Diagnostics* diag, struct file_block* fb );
	void readParticlePositions( Source* src, struct file_block* fb );
	void readParticleVelocities( Source* src, struct file_block* fb );

	std::string stringTrim(const std::string& pString, const std::string& pWhitespace = " \t");

public:
	InputFile();
	virtual ~InputFile();
	inline void setFileName(std::string inFilename) { filename = inFilename;};
	inline long long memory_footprint() { return sizeof(this) + vars.size()*sizeof(struct variable); };

	void read();
	struct file_block* get_block(struct file_block* fb, std::string name) ;
	bool getInputFileTestMode();
	std::vector<Species> getSpecies( std::string block_name );
	std::vector<Source> getSources( std::string block_name, std::vector<Species> &species );
	std::vector<InitialLoad> getInitLoad( std::string block_name, std::vector<Species> &species );
	SpatialRegion getSpatialRegion( std::string block_name );
	Tree getTree( std::string block_name );
	std::vector<Field> getFields( std::string block_name );
	std::vector<Diagnostics> getDiagnostics( std::string block_name, std::vector<Species> &species );

};

#endif /* INPUTFILE_H_ */
