/*
 * Simulation.h
 *
 *  Created on: Aug 7, 2011
 *      Author: janez
 */

#ifndef SIMULATION_H_
#define SIMULATION_H_

#include "peg_tc.h"

#include <vector>

#include "SpatialRegion.h"
#include "Species.h"
#include "ParticleList.h"
#include "InputFile.h"
#include "ParticleMover.h"
#include "Source.h"
#include "Progress.h"
#include "InitialLoad.h"
#include "Tree.h"
#include "Compute.h"
#include "Compute_green.h"
#include "Dump.h"
#include "Field.h"
#include "Diagnostics.h"

class Simulation {
	// data for simulation
	long int simStep, maxSimStep;
	bool defMaxSimStep, defdt;
	double t, dt;
	bool inputFileTest;
	std::string inputFileName, inputDumpFileName, outputDumpFilename ;
	InputFile inFile;
	Progress progress;
	SpatialRegion region;
	ParticleMover *mover;
	Compute *compute;
	Dump dump;
	Tree tree;
	ParticleList particles;
	std::vector<Species> species;
	std::vector<Source> sources;
	std::vector<InitialLoad> initLoad;
	std::vector<double> potential;
	std::vector<Field> fields;
	std::vector<Diagnostics> diags;

	long numParticlesRightOut;
	long numParticlesLeftOut;

	void writeDiagnostics();

public:
	Simulation();
	virtual ~Simulation();
	bool readInputFile(std::string inputFN);
	bool readFromDumpFile(std::string inDumpFilename);
	void init();
	int run();

	ParticleList* getParticleList() { return &particles; };
	SpatialRegion* getSpatialRegion() { return &region; };
	std::vector<double>* getPotential() { return &potential; };
	std::vector<Species>* getSpecies() { return &species; };
	std::vector<Field>* getFields() { return &fields; };

	void dispMemoryUsage(bool detail);
	std::string getMemoryUsage(bool detail);
	std::string getMemUsageNumberFormated(std::string text, long long total);

	long getNumParticlesRightOut() { return numParticlesRightOut; };
	long getNumParticlesLeftOut() { return numParticlesLeftOut; };

};

#endif /* SIMULATION_H_ */
