/*
 * SpatialRegion.h
 *
 *  Created on: Aug 7, 2011
 *      Author: janez
 */

#ifndef SPATIALREGION_H_
#define SPATIALREGION_H_

#include "peg_tc.h"

#include "Grid.h"
#include "ParticleList.h"
#include "Compute.h"

class SpatialRegion {
	double dt, area, p2c;
	double V0, V1;
	long int maxSimStep;
	Grid grid;
	int compute_method;

public:
	SpatialRegion();
	virtual ~SpatialRegion();
	void init();
	void dump(int level);
	bool defined(bool dispErrors);
	void checkBoundaries( ParticleList& pl );
	long countOutRight( ParticleList& pl );
	long countOutLeft( ParticleList& pl );

	inline void set_dt( double in_dt ) { dt = in_dt; };
	inline void set_area( double in_area ) { area = in_area; };
	inline void set_p2c( double in_p2c ) { p2c = in_p2c; };
	inline void set_maxStep( long int in_ns ) { maxSimStep = in_ns; };
	inline void set_grid( Grid in_grid ) { grid = in_grid; };
	inline void set_V0( double val ) { V0 = -val; };
	inline void set_V1( double val ) { V1 = -val; };
	inline void set_compute_method(int method) { compute_method = method; };

	inline double get_dt() { return dt; };
	inline double get_area() { return area; };
	inline double get_p2c() { return p2c; };
	inline long int get_maxStep() { return maxSimStep; };
	inline Grid get_grid() { return grid; };
	inline double get_V0() { return V0; };
	inline double get_V1() { return V1; };
	inline double get_grid_x_min() { return grid.get_r0().e1(); };
	inline double get_grid_x_max() { return grid.get_r1().e1(); };
	inline int get_compute_method() { return compute_method; };
};

#endif /* SPATIALREGION_H_ */
