/*
 * File taken from oopd1 simulation code and changed that it does not require any outside library
 * or definition. Removed all reference to ostring and integer versions of vectors.
 */

//====================================================================
#ifdef	COMMENTS

VECTOR.H

Class definitions for vectors, Vector3 and Vector2.  The Vector3 class is
a 3-component Vector3 representation with all the usual operators.  Vector2
is an implementation for 2-component vectors.  Note that all operations
are not valid on the latter.

The Vector3 class is a 3-component vector representation
encapsulating many of the standard mathematical vector operations.
Here 'vector' refers to the mathematical concept of a vector,
an ordered triplet representing a point in three-space.

The Vector2 class is a 2-component vector representation
encapsulating many of the standard mathematical vector operations.
Here 'vector' refers to the mathematical concept of a vector,
an ordered doublet representing a point in two-space.

Revision/Programmer/Date
0.1	(JohnV, 12-07-92)	Proto code.
0.2	(JohnV, 01-17-93) Fix case.
0.3	(JohnV, 02-20-93) Separate header file.
0.4	(JohnV, 02-27-93) Add Vector:: operator / (), -=() and +=().
0.5	(JohnV, 03-18-93)	Add Vector2 class.
0.6	(JohnV, 08-02-93) Inline constructors, eliminate VECTOR.CPP, add
Vector::operator *=(Vector&).
0.61	(JohnV, 11-22-93) Rename Vector->Vector3, add Vector3::norm(),
Vector3::scaleBy().
0.62	(JohnV NTG, 12-02-93) Removed friend function for c*Vector3,
absorbed Sqrt() global as Vector3::ComponentSqrt(), renamed
Vector3::norm() to magnitude().  Same changes for Vector2.
0.63	(JohnV 02-09-95) Added Vector3::operator /=.
0.64	(JohnV 05-28-95) Added isNonZero().
1.01	(PeterM 03-25-97) Segment class added.
1.02  (KeithC 06-08-97) added int vectors.

#endif
//====================================================================

#ifndef	 VECTOR_H_
#define	 VECTOR_H_

#include <math.h>

#define SCALAR double
#define Scalar SCALAR
#define sqr(x) ((x)*(x))


// THIS SHOULD BE DONE ELSEWHERE, JH, 5/20/2005
#define sqrt(x) sqrt((Scalar)(x))

// mindgame: How about n-dimensional vector template ?

class Vector3
{
	Scalar x1, x2, x3;

public:
	Vector3(Scalar _x1=0.0, Scalar _x2=0.0, Scalar _x3=0.0)
	{
		set(_x1, _x2, _x3);
	};

	int dim() {return 3;}

	inline Scalar	e1() const {return x1;};
	inline Scalar	e2() const {return x2;};
	inline Scalar	e3() const {return x3;};

	inline Scalar  e(int component) const
	{
		switch (component)
		{
		case 1:
			return x1;
		case 2:
			return x2;
		case 3:
			return x3;
		}
		return (0.0);
	}

	inline void	set_e1(Scalar _x1) {x1 = _x1;};
	inline void	set_e2(Scalar _x2) {x2 = _x2;};
	inline void	set_e3(Scalar _x3) {x3 = _x3;};
	inline void set(int component, Scalar value)
	{
		switch (component)
		{
		case 1:
			x1 = value;
			break;
		case 2:
			x2 = value;
			break;
		case 3:
			x3 = value;
			break;
		}
	}
	inline void set(Scalar _x1, Scalar _x2, Scalar _x3)
	{
		x1=_x1; x2=_x2; x3=_x3;
	}

	// dot product
	inline double dot(const Vector3& v) const
	{
		return x1*v.x1 + x2*v.x2 + x3*v.x3;
	};

	// cross product
	inline Vector3 cross(const Vector3& v) const
	{
		return Vector3(x2*v.x3-x3*v.x2, x3*v.x1-x1*v.x3, x1*v.x2-x2*v.x1);
	};

	/////////////////////////
	// overloaded operators
	/////////////////////////

	inline Vector3 operator + (const Vector3& v)  const
	{
		return Vector3(x1+v.x1, x2+v.x2, x3+v.x3);
	};

	inline Vector3 operator - (const Vector3& v) const
	{
		return Vector3(x1-v.x1, x2-v.x2, x3-v.x3);
	};

	inline Vector3 operator * (double c) const
			  {
		return Vector3(c*x1, c*x2, c*x3);
			  };

	inline Vector3 operator / (double c) const
			  {
		double ic = 1/c;
		return Vector3(ic*x1, ic*x2, ic*x3);
			  };

	inline Vector3 operator % (const Vector3& v) const
			  {
		//    return Vector3(x2*v.x3-x3*v.x2, x3*v.x1-x1*v.x3, x1*v.x2-x2*v.x1);
		return cross(v);
			  };

	inline double operator * (const Vector3& v) const
			  {
		return x1*v.x1 + x2*v.x2 + x3*v.x3;
			  };

	inline Vector3& operator += (double c)
			  {
		x1 += c; x2 += c; x3 += c; return *this;
			  };

	inline Vector3& operator -= (double c)
			  {
		x1 -= c; x2 -= c; x3 -= c; return *this;
			  };

	inline Vector3& operator *= (double c)
			  {
		x1 *= c; x2 *= c; x3 *= c; return *this;
			  };

	inline Vector3& operator *= (const Vector3& v)
			  {
		x1 *= v.x1; x2 *= v.x2; x3 *= v.x3;
		return *this;
			  };

	inline Vector3& operator += (const Vector3& v)
			  {
		x1 += v.x1; x2 += v.x2; x3 += v.x3;
		return *this;
			  };

	inline Vector3& operator -= (const Vector3& v)
			  {
		x1 -= v.x1; x2 -= v.x2; x3 -= v.x3;
		return *this;
			  };

	inline Vector3& operator /= (double c)
			  {
		Scalar ic = 1/c; return *this *= ic;
			  };

	inline Vector3& operator = (double c)
	{
		x1 = x2 = x3 = c; return *this;
	};


	Vector3 jvMult(const Vector3& v)
	{
		return Vector3(x1*v.x1, x2*v.x2, x3*v.x3);
	};

	Vector3 scaleBy(const Vector3 v) { return jvMult(v); };

	Vector3 jvDivide(const Vector3& v) //added by kc 8-30-94
			{
		return Vector3(x1/v.x1, x2/v.x2, x3/v.x3);
			};

	inline Scalar magnitude_squared(void)
	{
		return x1*x1 + x2*x2 + x3*x3;
	}
	inline Scalar magnitude(void)
	{
		return sqrt(magnitude_squared());
	};
	inline Vector3 unit(void)
	{
		double mag = magnitude();
		if (mag) {
			return Vector3(x1/mag, x2/mag, x3/mag);
		}
		else return Vector3(0.,0.,0.);
	}

	Vector3 ComponentSqrt(void)
	{
		return Vector3(sqrt(x1), sqrt(x2),	sqrt(x3));
	}

	Vector3 ComponentSqr(void)
	{
		return Vector3(sqr(x1), sqr(x2), sqr(x3));
	}

	inline bool isNonZero(void)
	{
		if (x1 == 0.0 && x2 == 0.0 && x3 == 0.) return 0;
		else return 1;
	}

	// returns angle between two vectors
	Scalar angle(Vector3 a)
	{
		Scalar dprod = dot(a);
		return acos(dprod*dprod/(a.magnitude_squared()*magnitude_squared()));
	}
};

inline Vector3	operator *(double c,const Vector3& v)
{
	return Vector3(c*v.e1(), c*v.e2(), c*v.e3());
}

inline bool operator == (const Vector3& v, const Vector3& u)
		{
	return(v.e1() == u.e1()  && v.e2() == u.e2()  && v.e3() == u.e3() );
		}





class Vector2
{
	Scalar	x1, x2;

public:
	Vector2(Scalar _x1=0.0, Scalar _x2=0.0) {x1=_x1; x2=_x2;};
	Vector2(const Vector2& v) {x1 = v.x1; x2 = v.x2;};
	int dim() {return 2;}
	Scalar e1() const {return x1;};
	Scalar e2() const  {return x2;};
	Scalar  e(int component) const{
		switch (component)
		{
		case 1:
			return x1;
		case 2:
			return x2;
		}
		return (0.0);
	}
	void	set_e1(Scalar _x1) {x1 = _x1;};
	void	set_e2(Scalar _x2) {x2 = _x2;};
	void set(int component, Scalar value){
		switch (component)
		{
		case 1:
			x1 = value;
			break;
		case 2:
			x2 = value;
			break;
		}
	}
	Vector2 operator + (const Vector2& v) const {return Vector2(x1+v.x1, x2+v.x2);};
	Vector2	operator - (const Vector2& v) const {return Vector2(x1-v.x1, x2-v.x2);};
	Vector2	operator * (double c) const {return Vector2(c*x1, c*x2);};
	Vector2	operator / (double c) const {Scalar ic = 1/c; return Vector2(ic*x1,
			ic*x2);};
	double	operator * (const Vector2& v) const {return x1*v.x1 + x2*v.x2;};
	Vector2&	operator += (double c) {x1 += c; x2 += c; return *this;};
	Vector2&	operator -= (double c) {x1 -= c; x2 -= c; return *this;};
	Vector2&	operator *= (double c) {x1 *= c; x2 *= c; return *this;};
	Vector2&	operator *= (const Vector2& v) {x1 *= v.x1; x2 *= v.x2;
	return *this;};
	Vector2&	operator += (const Vector2& v) {x1 += v.x1; x2 += v.x2; return *this;};
	Vector2&	operator -= (const Vector2& v) {x1 -= v.x1; x2 -= v.x2; return *this;};
	Vector2	jvMult(const Vector2& v) {return Vector2(x1*v.x1, x2*v.x2);};
	Vector2	jvDivide(const Vector2& v) {return Vector2(x1/v.x1, x2/v.x2);}; // added by kc 8-30-94
	Scalar	magnitude() {return sqrt(x1*x1 + x2*x2);};
	Vector2	ComponentSqrt(){return Vector2(sqrt(x1), sqrt(x2));}
	Vector2 ComponentSqr(){return Vector2(sqr(x1), sqr(x2));}
	inline bool	isNonZero() {if (x1 == 0.0 && x2 == 0) return 0;
	else return 1;}
};

inline Vector2	operator *(double c, const Vector2& v)
{
	return Vector2(c*v.e1(), c*v.e2());
}

class Vector1
{
	Scalar	x1;

public:
	Vector1(Scalar _x1=0.0) {x1=_x1;};
	Vector1(const Vector1& v) {x1 = v.x1;};
	int dim() {return 1;}
	Scalar e1() const {return x1;};
	Scalar  e(int component) const{
		if (component == 1) return x1;
		else return (0.0);
	}
	void	set_e1(Scalar _x1) {x1 = _x1;};
	void set(int component, Scalar value){
		if (component == 1)
			x1 = value;
	}
	Vector1 operator + (const Vector1& v) const {return Vector1(x1+v.x1);};
	Vector1	operator - (const Vector1& v) const {return Vector1(x1-v.x1);};
	Vector1	operator * (double c) const {return Vector1(c*x1);};
	Vector1	operator / (double c) const {Scalar ic = 1/c; return Vector1(ic*x1);};
	double	operator * (const Vector1& v) const {return x1*v.x1;};
	Vector1&	operator += (double c) {x1 += c; return *this;};
	Vector1&	operator -= (double c) {x1 -= c; return *this;};
	Vector1&	operator *= (double c) {x1 *= c; return *this;};
	Vector1&	operator *= (const Vector1& v) {x1 *= v.x1; return *this;};
	Vector1&	operator += (const Vector1& v) {x1 += v.x1; return *this;};
	Vector1&	operator -= (const Vector1& v) {x1 -= v.x1; return *this;};
	Vector1	jvMult(const Vector1& v) {return Vector1(x1*v.x1);};
	Vector1	jvDivide(const Vector1& v) {return Vector1(x1/v.x1);};
	Scalar	magnitude() {return fabs(x1);};
	Vector1	ComponentSqrt(){return Vector1(sqrt(x1));}
	Vector1 ComponentSqr(){return Vector1(sqr(x1));}
	inline bool	isNonZero() {if (x1 == 0.0) return 0;
	else return 1;}
};

inline Vector1	operator *(double c, const Vector1& v)
{
	return Vector1(c*v.e1());
}

#endif	//	ifndef OVECTOR_H_
