/**
 * ============================================================================
 * Name        : ltc.cpp (Lecad Tree Code)
 * Author      : Janez Krek, Nikola Jelic
 * Version     :
 * Copyright   : Copyright (c) LECAD 2011
 * Description : Plasma simulation code with treecode.
 * ============================================================================
 */

#include <iostream>
#include <string.h>
#include <stdlib.h>


#include "peg_tc.h"
#include "Simulation.h"
#include "Tools.h"

#include "RandomGenerator.h"

using namespace std;

#define		RUN_DISPLAY_USAGE	0
#define		RUN_SIMULATION		1


int main(int argc, char* argv[]) {
	int whatToDo = RUN_DISPLAY_USAGE;
	string inputFilename = "";
	string dumpFilename = "";


	if( argc < 2 ) {
		// no parameters; display usage
		whatToDo = RUN_DISPLAY_USAGE;
	} else {
		// check parameters in command line
		if( strncmp(argv[1], "-i", 2) == 0 ) {
			// input file; read and start simulation
			inputFilename = argv[2];
			whatToDo = RUN_SIMULATION;

		} else if( strncmp(argv[1], "-vdf", 4) == 0 ) {
			// get distribution function
			whatToDo = Tools::TOOLS_VDF;

		} else {
			// display usage
			whatToDo = RUN_DISPLAY_USAGE;
		}
	}

	switch( whatToDo ) {
	case RUN_SIMULATION: {
			// run simulation
			// create new simulation
			Simulation* sim = new Simulation();
			// read input file
			sim->readInputFile(inputFilename);

			if( dumpFilename != "" ) {
				// if we have dump file; read system state from dump file
				cout << "Reading dump file and starting simulation from dump file..." << endl;
				cout << "Not done yet!!" << endl;
				// sim->readFromDumpFile(dumpFilename);
			} else {
				// initialize the simulation -- set simulation to point zero (inject initial particles)
				sim->init();
			}

			// run the simulation
			sim->run();

			cout << endl << "Simulation done." << endl;
			break;
		}

	case Tools::TOOLS_VDF: {
			// run tools - get distribution functions
			Tools* tool = new Tools(Tools::TOOLS_VDF);

			tool->run( argc-2, argv+2 );
			delete tool;

			cout << endl << "Tools run done." << endl;
			break;
		}

	default:
	case RUN_DISPLAY_USAGE:
		// display usage and exit
		usage(argc, argv);
		exit(1);
		break;

	}


	return 0;
}


/**
 * Display usage.
 */
void usage(int argc, char* argv[]) {
	cout << "Usage: " << endl;
	cout << "\t" << argv[0] << " -i <input file>" << endl;
	cout << endl;
	return;
}
